;;; einfache-schematische-tiefensuche-2018.scm                    
(load "breakpoint.scm")
(load "ist-Euler-mit-gallenbacher-graph-kanten.scm")

;Das Programm muss noch vollstaendig ueberarbeitet werden:
;Bei Euler sind es Kanten!
;;; Loesung basiert auf:
;;; ===== schematische-Tiefensuche ==================================
;;; Die Tiefensuche ist zunaechst abstrakt definiert. Erst die
;;; jeweilige Implementation der Funktion Nachfolger und die
;;; Behandlung der Alternativen als Liste haengt von der Art
;;; der Repraesentation des Graphen ab.
;;; Es wird hier davon ausgegangen, dass dieser in einer
;;; Assoziationsliste vorliegt.
;;; Muster :
;;; ( (1.Knoten (1.Nachfolger 2.Nachfolger ...))
;;;   (2.Knoten (1.Nachfolger 2.Nachfolger ...))
;;;    ... )
;;;
;;; Die folgende Variante ermoeglicht nur eine Suche nach der
;;; ersten Loesung. Zyklen werden in dieser Version nicht abgefangen.
;;; ===== Tiefensuche ===============================================
; fuehrt eine Tiefensuche mit backtracking unter Verwendung
; des Systemstacks und einer besucht-Liste aus.
; Knoten x Knoten x Assoziationsliste --> boolean
; Die Ausgabe des Weges erfolgt aus der Liste besucht, wenn
; eine Suche erfolgreich war.
; Durch die interne Funktion wird das Übergeben von Start, Ziel
; und Graph vermieden.

;;; ----- speziell Eulerscher Weg -----
; Hier muss das Ziel gleich dem Start sein, daher muss ein Erfolg in
; jedem Fall vor dem Pruefen eines Zyklus untersucht und ausgegeben werden.

(define
  (Tiefensuche Start Graph)
  
  (define
    (t-s alternativen besuchte-kanten) ; bei Euler die Kanten

    ;(ausgabe besuchte-kanten #\newline)

    (cond

      ((null? alternativen) #f)                          ; [Anm.1]

      ((= (length besuchte-kanten) *anzahl*)

       (if 
        (equal? (second (first besuchte-kanten)) Start)  ; [Anm.2]
        besuchte-kanten          ; Loesung
        #f))                  ; keine Loesung

      ((kante-vorhanden? (first alternativen) besuchte-kanten)  ; [Anm.3]
       (t-s
        (rest alternativen)
        besuchte-kanten))

      ((t-s                                               ; [Anm.4]:
        (Nachfolger
         (second (first alternativen))
         Graph)
        (cons (first alternativen) besuchte-kanten))
       )

      (else
       (t-s                                        ; [Anm.5]
        (rest alternativen)
        besuchte-kanten))))
  
  (t-s (Nachfolger Start Graph) '()))

; [Anm.1]: Wenn auf einer Rekursionstiefe alle Alternativen
;   untersucht wurden, wird mit dem Wert "#f" zur aufrufenden
;   Ebene zurueckgesprungen.
; [Anm.2]: Wenn das Ziel erreicht wurde, wird die Kantenliste
;   zur aufrufenden Ebene zurueckgegeben.
;   Daher wird der Wert dort als "#t" behandelt.
; [Anm.3]: Es ist ein Zyklus aufgetreten, weiter suchen.
;   Hier muss aber auch nach der Gegenkante gesucht werden!
; [Anm.4]: Hier erfolgt der eigentliche Abstieg in der Rekursion.
;   Die aktuelle Kante wird in  besuchte-kanten aufgenommen.
; [Anm.5]: Wenn die Rekursion in der Tiefe erfolglos war, muss noch
;   die Rekursion fuer die Alternativen versucht werden.

;;; ===== Nachfolger ================================================
;;; Die Funktion Nachfolger ist zentral notwendig für die Loesung.
;;; Sie generiert jeweils zum aktuellen Knoten die Liste aller
;;; Nachfolgekanten.
(define
   (Nachfolger aktueller-knoten kanten)
  (cond
    ((null? kanten) '())
    ((equal? aktueller-knoten (first kanten))
     (cons
      (first kanten)
      (nachfolge-kanten aktueller-knoten (rest kanten))))
    (else (nachfolge-kanten aktueller-knoten (rest kanten)))
    )
  )
; Test:
;(Nachfolger 'I (kanten-raus '(I C 40) (kanten-raus '(O P 91) *kanten*)))
;(Nachfolger 'G  *kanten*)

;;; ----- vorhanden? ------------------------------------------------
; prueft, ob die Kante bereits besucht wurde
(define
  (kante-vorhanden? kante besuchte-kanten)
  (cond
    ((null? besuchte-kanten) #f)
    ((and
      (equal? (first kante) (first (first besuchte-kanten)))
      (equal? (second kante) (second (first besuchte-kanten))))
     #t)
    ((and
      (equal? (second kante) (first (first besuchte-kanten)))
      (equal? (first kante) (second (first besuchte-kanten))))
     #t)
    (else
     (kante-vorhanden? kante (rest besuchte-kanten)))))

;;; ===== Laufzeit ==================================================
(load "laufzeit-messung.scm")
(define Start 'I)
(define *anzahl* (/ (length *euler-graph*) 2))
(Zeit 'start)
(Tiefensuche Start *euler-graph*)
(Zeit 'stop)
(Zeit 'laenge)
