; ist-Euler mit gallenbacher-graph-kanten.scm
(load "breakpoint.scm")
;;; ===== ist-Euler-Graph? =====
; prueft, ob eine Kantenliste einen Euler-Graphen darstellt.
; Dazu wird untersucht, ob der Knotengrad immer gerade ist.
(define
  (ist-Euler-Graph? graph)
  (define
    (ist-intern? graph knoten grad)
    (cond
      ((null? graph)
       (even? grad)) ; auch letzten pruefen
      ((equal? knoten (first (first graph)))
       (ist-intern? (rest graph) knoten (+ grad 1)))
      ((even? grad) ; neuer Knoten
       (ist-intern? (rest graph) (first (first graph)) 1))
      (else #f)))
  (if (null? graph)
      (error "Graph darf nicht leer sein!")
      (ist-intern? (rest graph) (first (first graph)) 1)))
;;; ===== Kantenliste zum Gallenbacher-Graphen =====
; Stellt zum Gallenbacher-Graphen die Daten
(define
  *kanten*
  '((A B 69) (A D 36) (A M 36) (A N 22)
    (B A 69) (B D 64) (B Z 32) (B H 30) (B I 34) (B X 26)
    (C I 40) (C M 31) (C X 23)
    (D A 36) (D B 64) (D L 95) (D N 20)
    (E F 79) (E G 60) (E K 31) (E O 102)
    (F E 79) (F K 29) (F O 14) (F P 57)
    (G E 60) (G K 58) (G L 58) (G Y 30)
    (H B 30) (H I 65) (H K 31) (H L 106) (H P 34) (H Y 29)
    (I B 34) (I C 40) (I H 65) (I M 43) (I P 55)
    (K E 31) (K F 29) (K G 58) (K H 31) (K P 25) (K Y 20)
    (L D 95) (L G 58) (L H 106) (L Z 40)
    (M A 36) (M C 31) (M I 43) (M X 12)
    (N A 22) (N D 20) (N X 29) (N Z 30)
    (O E 102) (O F 14) (O P 91)
    (P H 34) (P F 57) (P I 55) (P K 25) (P O 91)
    (X B 26) (X C 23) (X M 12) (X N 29)
    (Y G 30) (Y H 29) (Y K 20) (Y Z 23)
    (Z B 32) (Z L 40) (Z N 30) (Z Y 23))
  )

;;; ===== kanten-raus =====
; entfernt die Kante und die Rueckkante aus der Kantenliste
(define
  (kanten-raus kante graph)
  (remove
   kante
   (remove
    (list (second kante) (first kante) (third kante))
    graph)))

; Tests:
;(ist-Euler-Graph? *kanten*)
;(kanten-raus '(I C 40) *kanten*)
;(ist-Euler-Graph? (kanten-raus '(I C 40) *kanten*))
;(kanten-raus '(I C 40) (kanten-raus '(O P 91) *kanten*))
;(ist-Euler-Graph? (kanten-raus '(I C 40) (kanten-raus '(O P 91) *kanten*)))

(define
  *euler-graph*
  '((A B 69) (A D 36) (A M 36) (A N 22)
    (B A 69) (B D 64) (B Z 32) (B H 30) (B I 34) (B X 26)
    (C M 31) (C X 23)
    (D A 36) (D B 64) (D L 95) (D N 20)
    (E F 79) (E G 60) (E K 31) (E O 102)
    (F E 79) (F K 29) (F O 14) (F P 57)
    (G E 60) (G K 58) (G L 58) (G Y 30)
    (H B 30) (H I 65) (H K 31) (H L 106) (H P 34) (H Y 29)
    (I B 34) (I H 65) (I M 43) (I P 55)
    (K E 31) (K F 29) (K G 58) (K H 31) (K P 25) (K Y 20)
    (L D 95) (L G 58) (L H 106) (L Z 40)
    (M A 36) (M C 31) (M I 43) (M X 12)
    (N A 22) (N D 20) (N X 29) (N Z 30)
    (O E 102) (O F 14)
    (P H 34) (P F 57) (P I 55) (P K 25)
    (X B 26) (X C 23) (X M 12) (X N 29)
    (Y G 30) (Y H 29) (Y K 20) (Y Z 23)
    (Z B 32) (Z L 40) (Z N 30) (Z Y 23))
  )


;;; ----- Nachfolger - Bereich -----
; ===== Hilfsfunktion ist-nachfolge-kante? =====
(define
  (ist-nachfolge-kante? aktuelle-stadt kante)
  (equal? aktuelle-stadt (first kante)))

; ===== nachfolge-kanten =====
; Muster fuer die Funktion Nachfolger
(define
  (nachfolge-kanten aktuelle-stadt kanten)
  (cond
    ((null? kanten) '())
    ((ist-nachfolge-kante? aktuelle-stadt (first kanten))
     (cons
      (first kanten)
      (nachfolge-kanten aktuelle-stadt (rest kanten))))
    (else (nachfolge-kanten aktuelle-stadt (rest kanten)))
    )
  )

